@@ -20,6 +20,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
 #pragma once
 
+#include "base/timer.h"
+
 namespace Storage {
 class Downloader;
 } // namespace Storage
@@ -30,6 +32,12 @@ class System;
 } // namespace Notifications
 } // namespace Window
 
+namespace Calls {
+class Instance;
+} // namespace Calls
+
+class ApiWrap;
+
 enum class EmojiPanelTab {
 	Emoji,
 	Stickers,
@@ -69,11 +77,32 @@ public:
 	void setEmojiPanelTab(EmojiPanelTab tab) {
 		_variables.emojiPanelTab = tab;
 	}
+	bool tabbedSelectorSectionEnabled() const {
+		return _variables.tabbedSelectorSectionEnabled;
+	}
+	void setTabbedSelectorSectionEnabled(bool enabled) {
+		_variables.tabbedSelectorSectionEnabled = enabled;
+	}
+	void setLastTimeVideoPlayedAt(TimeMs time) {
+		_lastTimeVideoPlayedAt = time;
+	}
+	TimeMs lastTimeVideoPlayedAt() const {
+		return _lastTimeVideoPlayedAt;
+	}
+	void setSoundOverride(const QString &key, const QString &path) {
+		_variables.soundOverrides.insert(key, path);
+	}
+	void clearSoundOverrides() {
+		_variables.soundOverrides.clear();
+	}
+	QString getSoundPath(const QString &key) const;
 
 private:
 	struct Variables {
 		bool lastSeenWarningSeen = false;
 		EmojiPanelTab emojiPanelTab = EmojiPanelTab::Emoji;
+		bool tabbedSelectorSectionEnabled = true;
+		QMap<QString, QString> soundOverrides;
 	};
 
 	base::Variable<bool> _contactsLoaded = { false };
@@ -81,10 +110,11 @@ private:
 	base::Observable<void> _moreChatsLoaded;
 	base::Observable<void> _savedGifsUpdated;
 	Variables _variables;
+	TimeMs _lastTimeVideoPlayedAt = 0;
 
 };
 
-class AuthSession final {
+class AuthSession final : private base::Subscriber {
 public:
 	AuthSession(UserId userId);
 
@@ -120,13 +150,31 @@ public:
 	AuthSessionData &data() {
 		return _data;
 	}
+	void saveDataDelayed(TimeMs delay);
+
+	ApiWrap &api() {
+		return *_api;
+	}
+
+	Calls::Instance &calls() {
+		return *_calls;
+	}
+
+	void checkAutoLock();
+	void checkAutoLockIn(TimeMs time);
 
 	~AuthSession();
 
 private:
-	UserId _userId = 0;
+	const UserId _userId = 0;
 	AuthSessionData _data;
+	base::Timer _saveDataTimer;
 
+	TimeMs _shouldLockAt = 0;
+	base::Timer _autoLockTimer;
+
+	const std::unique_ptr<ApiWrap> _api;
+	const std::unique_ptr<Calls::Instance> _calls;
 	const std::unique_ptr<Storage::Downloader> _downloader;
 	const std::unique_ptr<Window::Notifications::System> _notifications;
 
