@@ -25,8 +25,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "lang.h"
 #include "mainwidget.h"
 #include "mainwindow.h"
-#include "boxes/contactsbox.h"
-#include "boxes/confirmbox.h"
+#include "boxes/contacts_box.h"
+#include "boxes/confirm_box.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/scroll_area.h"
 #include "ui/effects/ripple_animation.h"
@@ -62,6 +62,12 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 //	return mapFromGlobal(QCursor::pos()) - _st.rippleAreaPosition;
 //}
 
+namespace {
+
+constexpr auto kReloadChannelAdminsTimeout = 1000; // 1 second wait before reload admins in channel after adding
+
+} // namespace
+
 MembersBox::MembersBox(QWidget*, ChannelData *channel, MembersFilter filter)
 : _channel(channel)
 , _filter(filter) {
@@ -124,7 +130,7 @@ void MembersBox::onAdminAdded() {
 	if (!_addBox) return;
 	_addBox->closeBox();
 	_addBox = nullptr;
-	_loadTimer->start(ReloadChannelMembersTimeout);
+	_loadTimer->start(kReloadChannelAdminsTimeout);
 }
 
 MembersBox::Inner::Inner(QWidget *parent, ChannelData *channel, MembersFilter filter) : TWidget(parent)
@@ -508,80 +514,81 @@ void MembersBox::Inner::onPeerNameChanged(PeerData *peer, const PeerData::Names
 }
 
 void MembersBox::Inner::membersReceived(const MTPchannels_ChannelParticipants &result, mtpRequestId req) {
+	Expects(result.type() == mtpc_channels_channelParticipants);
+
 	clear();
 	_loadingRequestId = 0;
 
-	if (result.type() == mtpc_channels_channelParticipants) {
-		auto &d = result.c_channels_channelParticipants();
-		auto &v = d.vparticipants.v;
-		_rows.reserve(v.size());
-		_datas.reserve(v.size());
-		_dates.reserve(v.size());
-		_roles.reserve(v.size());
+	auto &d = result.c_channels_channelParticipants();
+	auto &v = d.vparticipants.v;
+	_rows.reserve(v.size());
+	_datas.reserve(v.size());
+	_dates.reserve(v.size());
+	_roles.reserve(v.size());
 
-		if (_filter == MembersFilter::Recent && _channel->membersCount() < d.vcount.v) {
-			_channel->setMembersCount(d.vcount.v);
-			if (App::main()) emit App::main()->peerUpdated(_channel);
-		} else if (_filter == MembersFilter::Admins && _channel->adminsCount() < d.vcount.v) {
-			_channel->setAdminsCount(d.vcount.v);
-			if (App::main()) emit App::main()->peerUpdated(_channel);
+	if (_filter == MembersFilter::Recent && _channel->membersCount() < d.vcount.v) {
+		_channel->setMembersCount(d.vcount.v);
+		if (App::main()) emit App::main()->peerUpdated(_channel);
+	} else if (_filter == MembersFilter::Admins && _channel->adminsCount() < d.vcount.v) {
+		_channel->setAdminsCount(d.vcount.v);
+		if (App::main()) emit App::main()->peerUpdated(_channel);
+	}
+	App::feedUsers(d.vusers);
+
+	for (QVector<MTPChannelParticipant>::const_iterator i = v.cbegin(), e = v.cend(); i != e; ++i) {
+		int32 userId = 0, addedTime = 0;
+		MemberRole role = MemberRole::None;
+		switch (i->type()) {
+		case mtpc_channelParticipant:
+			userId = i->c_channelParticipant().vuser_id.v;
+			addedTime = i->c_channelParticipant().vdate.v;
+			break;
+		case mtpc_channelParticipantSelf:
+			role = MemberRole::Self;
+			userId = i->c_channelParticipantSelf().vuser_id.v;
+			addedTime = i->c_channelParticipantSelf().vdate.v;
+			break;
+		case mtpc_channelParticipantModerator:
+			role = MemberRole::Moderator;
+			userId = i->c_channelParticipantModerator().vuser_id.v;
+			addedTime = i->c_channelParticipantModerator().vdate.v;
+			break;
+		case mtpc_channelParticipantEditor:
+			role = MemberRole::Editor;
+			userId = i->c_channelParticipantEditor().vuser_id.v;
+			addedTime = i->c_channelParticipantEditor().vdate.v;
+			break;
+		case mtpc_channelParticipantKicked:
+			userId = i->c_channelParticipantKicked().vuser_id.v;
+			addedTime = i->c_channelParticipantKicked().vdate.v;
+			role = MemberRole::Kicked;
+			break;
+		case mtpc_channelParticipantCreator:
+			userId = i->c_channelParticipantCreator().vuser_id.v;
+			addedTime = _channel->date;
+			role = MemberRole::Creator;
+			break;
 		}
-		App::feedUsers(d.vusers);
-
-		for (QVector<MTPChannelParticipant>::const_iterator i = v.cbegin(), e = v.cend(); i != e; ++i) {
-			int32 userId = 0, addedTime = 0;
-			MemberRole role = MemberRole::None;
-			switch (i->type()) {
-			case mtpc_channelParticipant:
-				userId = i->c_channelParticipant().vuser_id.v;
-				addedTime = i->c_channelParticipant().vdate.v;
-				break;
-			case mtpc_channelParticipantSelf:
-				role = MemberRole::Self;
-				userId = i->c_channelParticipantSelf().vuser_id.v;
-				addedTime = i->c_channelParticipantSelf().vdate.v;
-				break;
-			case mtpc_channelParticipantModerator:
-				role = MemberRole::Moderator;
-				userId = i->c_channelParticipantModerator().vuser_id.v;
-				addedTime = i->c_channelParticipantModerator().vdate.v;
-				break;
-			case mtpc_channelParticipantEditor:
-				role = MemberRole::Editor;
-				userId = i->c_channelParticipantEditor().vuser_id.v;
-				addedTime = i->c_channelParticipantEditor().vdate.v;
-				break;
-			case mtpc_channelParticipantKicked:
-				userId = i->c_channelParticipantKicked().vuser_id.v;
-				addedTime = i->c_channelParticipantKicked().vdate.v;
-				role = MemberRole::Kicked;
-				break;
-			case mtpc_channelParticipantCreator:
-				userId = i->c_channelParticipantCreator().vuser_id.v;
-				addedTime = _channel->date;
-				role = MemberRole::Creator;
-				break;
-			}
-			if (UserData *user = App::userLoaded(userId)) {
-				_rows.push_back(user);
-				_dates.push_back(date(addedTime));
-				_roles.push_back(role);
-				_datas.push_back(0);
-			}
-		}
-
-		// update admins if we got all of them
-		if (_filter == MembersFilter::Admins && _channel->isMegagroup() && _rows.size() < Global::ChatSizeMax()) {
-			_channel->mgInfo->lastAdmins.clear();
-			for (int32 i = 0, l = _rows.size(); i != l; ++i) {
-				if (_roles.at(i) == MemberRole::Creator || _roles.at(i) == MemberRole::Editor) {
-					_channel->mgInfo->lastAdmins.insert(_rows.at(i));
-				}
-			}
-
-			Notify::peerUpdatedDelayed(_channel, Notify::PeerUpdate::Flag::AdminsChanged);
+		if (UserData *user = App::userLoaded(userId)) {
+			_rows.push_back(user);
+			_dates.push_back(date(addedTime));
+			_roles.push_back(role);
+			_datas.push_back(0);
 		}
 	}
+
+	// update admins if we got all of them
+	if (_filter == MembersFilter::Admins && _channel->isMegagroup() && _rows.size() < Global::ChatSizeMax()) {
+		_channel->mgInfo->lastAdmins.clear();
+		for (int32 i = 0, l = _rows.size(); i != l; ++i) {
+			if (_roles.at(i) == MemberRole::Creator || _roles.at(i) == MemberRole::Editor) {
+				_channel->mgInfo->lastAdmins.insert(_rows.at(i));
+			}
+		}
+
+		Notify::peerUpdatedDelayed(_channel, Notify::PeerUpdate::Flag::AdminsChanged);
+	}
+
 	if (_rows.isEmpty()) {
 		_rows.push_back(App::self());
 		_dates.push_back(date(MTP_int(_channel->date)));
