@@ -24,7 +24,14 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "codegen/emoji/generator.h"
 
 int main(int argc, char *argv[]) {
+#ifdef SUPPORT_IMAGE_GENERATION
+#ifndef Q_OS_MAC
+#error "Image generation is supported only on macOS"
+#endif // Q_OS_MAC
 	QGuiApplication app(argc, argv);
+#else // SUPPORT_IMAGE_GENERATION
+	QCoreApplication app(argc, argv);
+#endif // SUPPORT_IMAGE_GENERATION
 
 	auto options = codegen::emoji::parseOptions();
 
