@@ -21,6 +21,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "data/data_drafts.h"
 
 #include "ui/widgets/input_fields.h"
+#include "chat_helpers/message_field.h"
 #include "historywidget.h"
 #include "mainwidget.h"
 #include "storage/localstorage.h"
@@ -41,7 +42,7 @@ void applyPeerCloudDraft(PeerId peerId, const MTPDdraftMessage &draft) {
 	auto history = App::history(peerId);
 	auto text = qs(draft.vmessage);
 	auto entities = draft.has_entities() ? entitiesFromMTP(draft.ventities.v) : EntitiesInText();
-	TextWithTags textWithTags = { textApplyEntities(text, entities), textTagsFromEntities(entities) };
+	TextWithTags textWithTags = { textApplyEntities(text, entities), ConvertEntitiesToTextTags(entities) };
 	MsgId replyTo = draft.has_reply_to_msg_id() ? draft.vreply_to_msg_id.v : 0;
 	auto cloudDraft = std::make_unique<Draft>(textWithTags, replyTo, MessageCursor(QFIXED_MAX, QFIXED_MAX, QFIXED_MAX), draft.is_no_webpage());
 	cloudDraft->date = ::date(draft.vdate);
