@@ -113,6 +113,9 @@ public:
 	}
 	virtual void stopInline() {
 	}
+	virtual bool isRoundVideoPlaying() const {
+		return false;
+	}
 
 	virtual void attachToParent() {
 	}
@@ -151,6 +154,13 @@ public:
 		return false;
 	}
 
+	// An attach media in a web page can provide an
+	// additional text to be displayed below the attach.
+	// For example duration / progress for video messages.
+	virtual QString additionalInfoString() const {
+		return QString();
+	}
+
 	int currentWidth() const {
 		return _width;
 	}
@@ -171,6 +181,10 @@ public:
 		return false;
 	}
 
+	virtual bool canEditCaption() const {
+		return false;
+	}
+
 	// Sometimes click on media in message is overloaded by the messsage:
 	// (for example it can open a link or a game instead of opening media)
 	// But the overloading click handler should be used only when media
