@@ -18,11 +18,10 @@ to link the code of portions of this program with the OpenSSL library.
 Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
 Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
-#include "stdafx.h"
 #include "inline_bots/inline_bot_send_data.h"
 
 #include "inline_bots/inline_bot_result.h"
-#include "localstorage.h"
+#include "storage/localstorage.h"
 
 namespace InlineBots {
 namespace internal {
@@ -38,8 +37,8 @@ QString SendData::getLayoutDescription(const Result *owner) const {
 void SendDataCommon::addToHistory(const Result *owner, History *history,
 MTPDmessage::Flags flags, MsgId msgId, UserId fromId, MTPint mtpDate,
 UserId viaBotId, MsgId replyToId, const MTPReplyMarkup &markup) const {
-	SentMTPMessageFields fields = getSentMessageFields();
-	if (!fields.entities.c_vector().v.isEmpty()) {
+	auto fields = getSentMessageFields();
+	if (!fields.entities.v.isEmpty()) {
 		flags |= MTPDmessage::Flag::f_entities;
 	}
 	history->addNewMessage(MTP_message(MTP_flags(flags), MTP_int(msgId), MTP_int(fromId), peerToMTP(history->peer->id), MTPnullFwdHeader, MTP_int(viaBotId), MTP_int(replyToId), mtpDate, fields.text, fields.media, markup, fields.entities, MTP_int(1), MTPint()), NewMessageUnread);
