@@ -28,7 +28,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "ui/widgets/labels.h"
 #include "ui/effects/widget_fade_wrap.h"
 #include "core/click_handler_types.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "messenger.h"
 
 namespace Intro {
@@ -249,7 +249,7 @@ void PhoneWidget::activate() {
 void PhoneWidget::finished() {
 	Step::finished();
 	_checkRequest->stop();
-	rpcClear();
+	rpcInvalidate();
 
 	cancelled();
 }
