@@ -23,8 +23,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "styles/style_intro.h"
 #include "styles/style_boxes.h"
 #include "core/file_utilities.h"
-#include "boxes/photocropbox.h"
-#include "boxes/confirmbox.h"
+#include "boxes/photo_crop_box.h"
+#include "boxes/confirm_box.h"
 #include "lang.h"
 #include "application.h"
 #include "ui/widgets/buttons.h"
