@@ -32,7 +32,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "mainwindow.h"
 #include "messenger.h"
 #include "application.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "ui/text/text.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/labels.h"
@@ -121,7 +121,6 @@ void Widget::changeLanguage(int32 languageId) {
 }
 
 void Widget::setInnerFocus() {
-	Global::RefDialogsListFocused().set(false, true);
 	if (getStep()->animating()) {
 		setFocus();
 	} else {
@@ -154,20 +153,12 @@ void Widget::historyMove(Direction direction) {
 	if (direction == Direction::Back || direction == Direction::Replace) {
 		delete base::take(wasStep);
 	}
-	if (getStep()->hasBack()) {
-		_back->showAnimated();
-	} else {
-		_back->hideAnimated();
-	}
-	if (getStep()->hasCover()) {
-		_settings->hideAnimated();
-		if (_update) _update->hideAnimated();
-		if (_changeLanguage) _changeLanguage->showAnimated();
-	} else {
-		_settings->showAnimated();
-		if (_update) _update->showAnimated();
-		if (_changeLanguage) _changeLanguage->hideAnimated();
-	}
+	_back->toggleAnimated(getStep()->hasBack());
+
+	auto stepHasCover = getStep()->hasCover();
+	_settings->toggleAnimated(!stepHasCover);
+	if (_update) _update->toggleAnimated(!stepHasCover);
+	if (_changeLanguage) _changeLanguage->toggleAnimated(stepHasCover);
 	_next->setText(getStep()->nextButtonText());
 	if (_resetAccount) _resetAccount->hideAnimated();
 	getStep()->showAnimated(direction);
