@@ -20,6 +20,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
 #pragma once
 
+#include "lang_auto.h"
+
 constexpr const str_const LanguageCodes[] = {
 	"en",
 	"it",
@@ -31,53 +33,6 @@ constexpr const str_const LanguageCodes[] = {
 };
 constexpr const int languageTest = -1, languageDefault = 0, languageCount = base::array_size(LanguageCodes);
 
-class LangString : public QString {
-public:
-	LangString() {
-	}
-	LangString(const QString &str) : QString(str) {
-	}
-
-	LangString tag(ushort tag, const QString &replacement) {
-		for (const QChar *s = constData(), *ch = s, *e = ch + size(); ch != e;) {
-			if (*ch == TextCommand) {
-				if (ch + 3 < e && (ch + 1)->unicode() == TextCommandLangTag && *(ch + 3) == TextCommand) {
-					if ((ch + 2)->unicode() == 0x0020 + tag) {
-						LangString result;
-						result.reserve(size() + replacement.size() - 4);
-						if (ch > s) result.append(midRef(0, ch - s));
-						result.append(replacement);
-						if (ch + 4 < e) result.append(midRef(ch - s + 4));
-						return result;
-					} else {
-						ch += 4;
-					}
-				} else {
-					const QChar *next = textSkipCommand(ch, e);
-					if (next == ch) {
-						++ch;
-					} else {
-						ch = next;
-					}
-				}
-			} else {
-				++ch;
-			}
-		}
-		return *this;
-	}
-
-	LangString &operator=(const QString &str) {
-		QString::operator=(str);
-		return (*this);
-	}
-
-};
-
-LangString langCounted(ushort key0, ushort tag, float64 value);
-
-#include "lang_auto.h"
-
 const char *langKeyName(LangKey key);
 
 template <typename WithYear, typename WithoutYear>
@@ -187,8 +142,8 @@ protected:
 		memset(_found, 0, sizeof(_found));
 	}
 
-	ushort tagIndex(const QByteArray &tag) const;
-	LangKey keyIndex(const QByteArray &key) const;
+	ushort tagIndex(QLatin1String tag) const;
+	LangKey keyIndex(QLatin1String key) const;
 	bool tagReplaced(LangKey key, ushort tag) const;
 	LangKey subkeyIndex(LangKey key, ushort tag, ushort index) const;
 
