@@ -21,7 +21,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "lang.h"
 
 #include "media/media_clip_reader.h"
-#include "boxes/abstractbox.h"
+#include "boxes/abstract_box.h"
 #include "layerwidget.h"
 #include "application.h"
 #include "mainwindow.h"
@@ -29,10 +29,11 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "core/file_utilities.h"
 #include "styles/style_boxes.h"
 #include "styles/style_widgets.h"
-#include "styles/style_stickers.h"
+#include "styles/style_chat_helpers.h"
 #include "ui/widgets/shadow.h"
 #include "window/window_main_menu.h"
 #include "auth_session.h"
+#include "window/window_controller.h"
 
 namespace {
 
@@ -311,7 +312,8 @@ void LayerStackWidget::BackgroundWidget::animationCallback() {
 	checkIfDone();
 }
 
-LayerStackWidget::LayerStackWidget(QWidget *parent) : TWidget(parent)
+LayerStackWidget::LayerStackWidget(QWidget *parent, Window::Controller *controller) : TWidget(parent)
+, _controller(controller)
 , _background(this) {
 	setGeometry(parentWidget()->rect());
 	hide();
@@ -537,7 +539,6 @@ void LayerStackWidget::showBox(object_ptr<BoxContent> box) {
 void LayerStackWidget::prepareForAnimation() {
 	if (isHidden()) {
 		show();
-		App::wnd()->enableGifPauseReason(Window::GifPauseReason::Layer);
 	}
 	if (_mainMenu) {
 		_mainMenu->hide();
@@ -566,7 +567,6 @@ void LayerStackWidget::animationDone() {
 	}
 	if (hidden) {
 		App::wnd()->layerFinishedHide(this);
-		App::wnd()->disableGifPauseReason(Window::GifPauseReason::Layer);
 	} else {
 		showFinished();
 	}
@@ -623,7 +623,7 @@ LayerWidget *LayerStackWidget::pushBox(object_ptr<BoxContent> box) {
 		if (oldLayer->inFocusChain()) setFocus();
 		oldLayer->hide();
 	}
-	auto layer = object_ptr<AbstractBox>(this, std::move(box));
+	auto layer = object_ptr<AbstractBox>(this, _controller, std::move(box));
 	_layers.push_back(layer);
 	initChildLayer(layer);
 
@@ -645,7 +645,7 @@ void LayerStackWidget::prependBox(object_ptr<BoxContent> box) {
 	if (_layers.empty()) {
 		return showBox(std::move(box));
 	}
-	auto layer = object_ptr<AbstractBox>(this, std::move(box));
+	auto layer = object_ptr<AbstractBox>(this, _controller, std::move(box));
 	layer->hide();
 	_layers.push_front(layer);
 	initChildLayer(layer);
@@ -720,7 +720,8 @@ LayerStackWidget::~LayerStackWidget() {
 	if (App::wnd()) App::wnd()->noLayerStack(this);
 }
 
-MediaPreviewWidget::MediaPreviewWidget(QWidget *parent) : TWidget(parent)
+MediaPreviewWidget::MediaPreviewWidget(QWidget *parent, gsl::not_null<Window::Controller*> controller) : TWidget(parent)
+, _controller(controller)
 , _emojiSize(Ui::Emoji::Size(Ui::Emoji::Index() + 1) / cIntRetinaFactor()) {
 	setAttribute(Qt::WA_TransparentForMouseEvents);
 	subscribe(AuthSession::CurrentDownloaderTaskFinished(), [this] { update(); });
@@ -736,7 +737,7 @@ void MediaPreviewWidget::paintEvent(QPaintEvent *e) {
 	if (!_a_shown.animating()) {
 		if (_hiding) {
 			hide();
-			App::wnd()->disableGifPauseReason(Window::GifPauseReason::MediaPreview);
+			_controller->disableGifPauseReason(Window::GifPauseReason::MediaPreview);
 			return;
 		}
 	} else {
@@ -763,7 +764,7 @@ void MediaPreviewWidget::resizeEvent(QResizeEvent *e) {
 }
 
 void MediaPreviewWidget::showPreview(DocumentData *document) {
-	if (!document || (!document->isAnimation() && !document->sticker())) {
+	if (!document || (!document->isAnimation() && !document->sticker()) || document->isRoundVideo()) {
 		hidePreview();
 		return;
 	}
@@ -793,7 +794,7 @@ void MediaPreviewWidget::startShow() {
 	if (isHidden() || _a_shown.animating()) {
 		if (isHidden()) {
 			show();
-			App::wnd()->enableGifPauseReason(Window::GifPauseReason::MediaPreview);
+			_controller->enableGifPauseReason(Window::GifPauseReason::MediaPreview);
 		}
 		_hiding = false;
 		_a_shown.start([this] { update(); }, 0., 1., st::stickerPreviewDuration);
@@ -927,7 +928,7 @@ QPixmap MediaPreviewWidget::currentImage() const {
 			}
 			if (_gif && _gif->started()) {
 				auto s = currentDimensions();
-				auto paused = App::wnd()->isGifPausedAtLeastFor(Window::GifPauseReason::MediaPreview);
+				auto paused = _controller->isGifPausedAtLeastFor(Window::GifPauseReason::MediaPreview);
 				return _gif->current(s.width(), s.height(), s.width(), s.height(), ImageRoundRadius::None, ImageRoundCorner::None, paused ? 0 : getms());
 			}
 			if (_cacheStatus != CacheThumbLoaded && _document->thumb->loaded()) {
