@@ -43,6 +43,7 @@ namespace Theme {
 struct BackgroundUpdate;
 class WarningWidget;
 } // namespace Theme
+class Controller;
 } // namespace Window
 
 namespace Ui {
@@ -78,6 +79,10 @@ public:
 	MainWindow();
 	~MainWindow();
 
+	Window::Controller *controller() const {
+		return _controller.get();
+	}
+
 	void firstShow();
 
 	void inactivePress(bool inactive);
@@ -85,7 +90,6 @@ public:
 
 	void setupPasscode();
 	void clearPasscode();
-	void checkAutoLockIn(int msec);
 	void setupIntro();
 	void setupMain(const MTPUser *user = nullptr);
 	void serviceNotification(const TextWithEntities &message, const MTPMessageMedia &media = MTP_messageMediaEmpty(), int32 date = 0, bool force = false);
@@ -103,7 +107,6 @@ public:
 	void activate();
 
 	void noIntro(Intro::Widget *was);
-	void noMain(MainWidget *was);
 	void noLayerStack(LayerStackWidget *was);
 	void layerFinishedHide(LayerStackWidget *was);
 
@@ -120,8 +123,6 @@ public:
 	TempDirState localStorageState();
 	void tempDirDelete(int task);
 
-	QImage iconLarge() const;
-
 	void sendPaths();
 
 	void changingMsgId(HistoryItem *row, MsgId newId) override;
@@ -160,8 +161,6 @@ protected:
 	void updateControlsGeometry() override;
 
 public slots:
-	void checkAutoLock();
-
 	void showSettings();
 	void layerHidden();
 	void setInnerFocus();
@@ -195,9 +194,13 @@ private slots:
 	void onWindowActiveChanged();
 
 private:
+	void checkAuthSession();
 	void showConnecting(const QString &text, const QString &reconnect = QString());
 	void hideConnecting();
 
+	void ensureLayerCreated();
+	void destroyLayerDelayed();
+
 	void themeUpdated(const Window::Theme::BackgroundUpdate &data);
 
 	QPixmap grabInner();
@@ -215,6 +218,7 @@ private:
 	QList<DelayedServiceMsg> _delayedServiceMsgs;
 	mtpRequestId _serviceHistoryRequest = 0;
 
+	std::unique_ptr<Window::Controller> _controller;
 	object_ptr<PasscodeWidget> _passcode = { nullptr };
 	object_ptr<Intro::Widget> _intro = { nullptr };
 	object_ptr<MainWidget> _main = { nullptr };
@@ -229,9 +233,6 @@ private:
 	bool _inactivePress = false;
 	QTimer _inactiveTimer;
 
-	SingleTimer _autoLockTimer;
-	TimeMs _shouldLockAt = 0;
-
 };
 
 class PreLaunchWindow : public TWidget {
