@@ -26,6 +26,28 @@ struct VideoSoundData;
 struct VideoSoundPart;
 
 namespace Media {
+namespace Audio {
+
+// Thread: Main.
+void Start();
+void Finish();
+
+// Thread: Main. Locks: AudioMutex.
+bool IsAttachedToDevice();
+
+// Thread: Any. Must be locked: AudioMutex.
+bool AttachToDevice();
+
+// Thread: Any.
+void ScheduleDetachFromDeviceSafe();
+void ScheduleDetachIfNotUsedSafe();
+void StopDetachIfNotUsedSafe();
+
+template <typename Callback>
+void IterateSamples();
+
+} // namespace Audio
+
 namespace Player {
 
 constexpr auto kDefaultFrequency = 48000; // 48 kHz
@@ -35,13 +57,7 @@ constexpr auto kWaveformSamplesCount = 100;
 class Fader;
 class Loaders;
 
-void InitAudio();
-void DeInitAudio();
-
 base::Observable<AudioMsgId> &Updated();
-void DetachFromDeviceByTimer();
-
-void PlayNotify();
 
 float64 ComputeVolume(AudioMsgId::Type type);
 
@@ -87,7 +103,7 @@ struct TrackState {
 	AudioMsgId id;
 	State state = State::Stopped;
 	int64 position = 0;
-	TimeMs duration = 0;
+	int64 length = 0;
 	int frequency = kDefaultFrequency;
 };
 
@@ -117,11 +133,19 @@ public:
 
 	void clearStoppedAtStart(const AudioMsgId &audio);
 
-	void detachFromDeviceByTimer();
+	// Thread: Main. Must be locked: AudioMutex.
 	void detachTracks();
+
+	// Thread: Main. Must be locked: AudioMutex.
 	void reattachIfNeeded();
+
+	// Thread: Any. Must be locked: AudioMutex.
 	void reattachTracks();
 
+	// Thread: Any.
+	void setVideoVolume(float64 volume);
+	float64 getVideoVolume() const;
+
 	~Mixer();
 
 private slots:
@@ -140,7 +164,7 @@ signals:
 
 	void suppressSong();
 	void unsuppressSong();
-	void suppressAll();
+	void suppressAll(qint64 duration);
 
 private:
 	bool fadedStop(AudioMsgId::Type type, bool *fadedStart = 0);
@@ -208,6 +232,7 @@ private:
 	Track _songTracks[kTogetherLimit];
 
 	Track _videoTrack;
+	QAtomicInt _videoVolume;
 	uint64 _lastVideoPlayId = 0;
 	TimeMs _lastVideoPlaybackWhen = 0;
 	TimeMs _lastVideoPlaybackCorrectedMs = 0;
@@ -231,7 +256,6 @@ class Fader : public QObject {
 
 public:
 	Fader(QThread *thread);
-	void keepAttachedToDevice();
 
 signals:
 	void error(const AudioMsgId &audio);
@@ -240,15 +264,12 @@ signals:
 	void needToPreload(const AudioMsgId &audio);
 
 public slots:
-	void onDetachFromDeviceByTimer(bool force);
-
 	void onInit();
 	void onTimer();
-	void onDetachFromDeviceTimer();
 
 	void onSuppressSong();
 	void onUnsuppressSong();
-	void onSuppressAll();
+	void onSuppressAll(qint64 duration);
 	void onSongVolumeChanged();
 	void onVideoVolumeChanged();
 
@@ -272,27 +293,54 @@ private:
 	bool _videoVolumeChanged = false;
 	anim::value _suppressAllGain, _suppressSongGain;
 	TimeMs _suppressAllStart = 0;
+	TimeMs _suppressAllEnd = 0;
 	TimeMs _suppressSongStart = 0;
 
-	QTimer _detachFromDeviceTimer;
-	QMutex _detachFromDeviceMutex;
-	bool _detachFromDeviceForce = false;
-
 };
 
 FileLoadTask::Song PrepareForSending(const QString &fname, const QByteArray &data);
 
-} // namespace Player
-} // namespace Media
-
 namespace internal {
 
-QMutex *audioPlayerMutex();
-bool audioCheckError();
-
-// AudioMutex must be locked.
+// Thread: Any. Must be locked: AudioMutex.
 bool CheckAudioDeviceConnected();
 
+// Thread: Main. Locks: AudioMutex.
+void DetachFromDevice();
+
+// Thread: Any.
+QMutex *audioPlayerMutex();
+
+// Thread: Any.
+bool audioCheckError();
+
 } // namespace internal
 
+} // namespace Player
+} // namespace Media
+
 VoiceWaveform audioCountWaveform(const FileLocation &file, const QByteArray &data);
+
+namespace Media {
+namespace Audio {
+
+FORCE_INLINE uint16 ReadOneSample(uchar data) {
+	return qAbs((static_cast<int16>(data) - 0x80) * 0x100);
+}
+
+FORCE_INLINE uint16 ReadOneSample(int16 data) {
+	return qAbs(data);
+}
+
+template <typename SampleType, typename Callback>
+void IterateSamples(base::const_byte_span bytes, Callback &&callback) {
+	auto samplesPointer = reinterpret_cast<const SampleType*>(bytes.data());
+	auto samplesCount = bytes.size() / sizeof(SampleType);
+	auto samplesData = gsl::make_span(samplesPointer, samplesCount);
+	for (auto sampleData : samplesData) {
+		callback(ReadOneSample(sampleData));
+	}
+}
+
+} // namespace Audio
+} // namespace Media
