@@ -128,7 +128,9 @@ Widget::Widget(QWidget *parent) : TWidget(parent)
 		handlePlaylistUpdate();
 	});
 	subscribe(instance()->updatedNotifier(), [this](const TrackState &state) {
-		handleSongUpdate(state);
+		if (state.id.type() == AudioMsgId::Type::Song) {
+			handleSongUpdate(state);
+		}
 	});
 	subscribe(instance()->songChangedNotifier(), [this] {
 		handleSongChange();
@@ -206,8 +208,8 @@ void Widget::handleSeekFinished(float64 progress) {
 
 	auto type = AudioMsgId::Type::Song;
 	auto state = mixer()->currentState(type);
-	if (state.id && state.duration) {
-		mixer()->seek(type, qRound(progress * state.duration));
+	if (state.id && state.length) {
+		mixer()->seek(type, qRound(progress * state.length));
 	}
 
 	instance()->stopSeeking(AudioMsgId::Type::Song);
@@ -299,7 +301,7 @@ void Widget::updateRepeatTrackIcon() {
 }
 
 void Widget::handleSongUpdate(const TrackState &state) {
-	if (!state.id.audio() || !state.id.audio()->song()) {
+	if (!state.id.audio()) {
 		return;
 	}
 
@@ -329,16 +331,18 @@ void Widget::handleSongUpdate(const TrackState &state) {
 
 void Widget::updateTimeText(const TrackState &state) {
 	QString time;
-	qint64 position = 0, duration = 0, display = 0;
+	qint64 position = 0, length = 0, display = 0;
 	auto frequency = state.frequency;
 	if (!IsStopped(state.state) && state.state != State::Finishing) {
 		display = position = state.position;
-		duration = state.duration;
-	} else {
-		display = state.duration ? state.duration : (state.id.audio()->song()->duration * frequency);
+		length = state.length;
+	} else if (state.length) {
+		display = state.length;
+	} else if (state.id.audio()->song()) {
+		display = (state.id.audio()->song()->duration * frequency);
 	}
 
-	_lastDurationMs = (state.duration * 1000LL) / frequency;
+	_lastDurationMs = (state.length * 1000LL) / frequency;
 
 	if (state.id.audio()->loading()) {
 		_time = QString::number(qRound(state.id.audio()->progress() * 100)) + '%';
@@ -369,13 +373,10 @@ void Widget::updateTimeLabel() {
 void Widget::handleSongChange() {
 	auto &current = instance()->current();
 	auto song = current.audio()->song();
-	if (!song) {
-		return;
-	}
 
 	TextWithEntities textWithEntities;
-	if (song->performer.isEmpty()) {
-		textWithEntities.text = song->title.isEmpty() ? (current.audio()->name.isEmpty() ? qsl("Unknown Track") : current.audio()->name) : song->title;
+	if (!song || song->performer.isEmpty()) {
+		textWithEntities.text = (!song || song->title.isEmpty()) ? (current.audio()->name.isEmpty() ? qsl("Unknown Track") : current.audio()->name) : song->title;
 	} else {
 		auto title = song->title.isEmpty() ? qsl("Unknown Track") : textClean(song->title);
 		textWithEntities.text = song->performer + QString::fromUtf8(" \xe2\x80\x93 ") + title;
