@@ -68,7 +68,7 @@ private:
 	Ui::ContinuousSlider *_slider;
 
 	int64 _position = 0;
-	TimeMs _duration = 0;
+	int64 _length = 0;
 
 	bool _playing = false;
 
