@@ -20,7 +20,11 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
 #pragma once
 
-#include "core/observer.h"
+#include "base/observer.h"
+
+namespace App {
+void quit();
+} // namespace App
 
 namespace MTP {
 class DcOptions;
@@ -28,10 +32,21 @@ class Instance;
 } // namespace MTP
 
 class AuthSession;
+class AuthSessionData;
 class MainWidget;
 class FileUploader;
 class Translator;
 
+namespace Local {
+struct StoredAuthSession;
+} // namespace Local
+
+namespace Media {
+namespace Audio {
+class Instance;
+} // namespace Audio
+} // namespace Media
+
 class Messenger final : public QObject, public RPCSender, private base::Subscriber {
 	Q_OBJECT
 
@@ -41,10 +56,16 @@ public:
 	Messenger(const Messenger &other) = delete;
 	Messenger &operator=(const Messenger &other) = delete;
 
-	void prepareToDestroy();
 	~Messenger();
 
 	MainWindow *mainWindow();
+	QPoint getPointForCallPanelCenter() const;
+	QImage logo() const {
+		return _logo;
+	}
+	QImage logoNoMargin() const {
+		return _logoNoMargin;
+	}
 
 	static Messenger *InstancePointer();
 	static Messenger &Instance() {
@@ -53,14 +74,22 @@ public:
 		return *result;
 	}
 
+	// MTProto components.
 	MTP::DcOptions *dcOptions() {
 		return _dcOptions.get();
 	}
 
+	// Set from legacy storage.
 	void setMtpMainDcId(MTP::DcId mainDcId);
 	void setMtpKey(MTP::DcId dcId, const MTP::AuthKey::Data &keyData);
+	void setAuthSessionUserId(UserId userId);
+	void setAuthSessionFromStorage(std::unique_ptr<Local::StoredAuthSession> data);
+	AuthSessionData *getAuthSessionData();
+
+	// Serialization.
 	QByteArray serializeMtpAuthorization() const;
 	void setMtpAuthorization(const QByteArray &serialized);
+
 	void startMtp();
 	MTP::Instance *mtp() {
 		return _mtproto.get();
@@ -68,6 +97,7 @@ public:
 	void suggestMainDcId(MTP::DcId mainDcId);
 	void destroyStaleAuthorizationKeys();
 
+	// AuthSession component.
 	AuthSession *authSession() {
 		return _authSession.get();
 	}
@@ -77,6 +107,11 @@ public:
 		return _authSessionChanged;
 	}
 
+	// Media component.
+	Media::Audio::Instance &audio() {
+		return *_audio;
+	}
+
 	void setInternalLinkDomain(const QString &domain) const;
 	QString createInternalLink(const QString &query) const;
 	QString createInternalLinkFull(const QString &query) const;
@@ -96,17 +131,26 @@ public:
 
 	void writeUserConfigIn(TimeMs ms);
 
-	void killDownloadSessionsStart(int32 dc);
-	void killDownloadSessionsStop(int32 dc);
+	void killDownloadSessionsStart(MTP::DcId dcId);
+	void killDownloadSessionsStop(MTP::DcId dcId);
 
 	void checkLocalTime();
 	void checkMapVersion();
+	void setupPasscode();
+	void clearPasscode();
+	base::Observable<void> &passcodedChanged() {
+		return _passcodedChanged;
+	}
+
+	void quitPreventFinished();
 
 	void handleAppActivated();
 	void handleAppDeactivated();
 
-	// Temporary here, when all Images and Documents are owned by AuthSession it'll have this.
-	void delayedDestroyLoader(std::unique_ptr<FileLoader> loader);
+	void call_handleHistoryUpdate();
+	void call_handleUnreadCounterUpdate();
+	void call_handleDelayedPeerUpdates();
+	void call_handleObservables();
 
 signals:
 	void peerPhotoDone(PeerId peer);
@@ -114,7 +158,6 @@ signals:
 
 public slots:
 	void onAllKeysDestroyed();
-	void onDelayedDestroyLoaders();
 
 	void photoUpdated(const FullMsgId &msgId, bool silent, const MTPInputFile &file);
 
@@ -125,19 +168,18 @@ public slots:
 	void killDownloadSessions();
 	void onAppStateChanged(Qt::ApplicationState state);
 
-	void call_handleHistoryUpdate();
-	void call_handleUnreadCounterUpdate();
-	void call_handleDelayedPeerUpdates();
-	void call_handleObservables();
-
 private:
 	void destroyMtpKeys(MTP::AuthKeysList &&keys);
 	void startLocalStorage();
 	void loadLanguage();
 
+	friend void App::quit();
+	static void QuitAttempt();
+	void quitDelayed();
+
 	QMap<FullMsgId, PeerId> photoUpdates;
 
-	QMap<int32, TimeMs> killDownloadSessionTimes;
+	QMap<MTP::DcId, TimeMs> killDownloadSessionTimes;
 	SingleTimer killDownloadSessionsTimer;
 
 	// Some fields are just moved from the declaration.
@@ -146,15 +188,17 @@ private:
 
 	std::unique_ptr<MainWindow> _window;
 	FileUploader *_uploader = nullptr;
-	Translator *_translator = nullptr;
 
+	std::unique_ptr<Translator> _translator;
 	std::unique_ptr<MTP::DcOptions> _dcOptions;
 	std::unique_ptr<MTP::Instance> _mtproto;
 	std::unique_ptr<MTP::Instance> _mtprotoForKeysDestroy;
 	std::unique_ptr<AuthSession> _authSession;
 	base::Observable<void> _authSessionChanged;
+	base::Observable<void> _passcodedChanged;
 
-	SingleDelayedCall _delayedLoadersDestroyer;
-	std::vector<std::unique_ptr<FileLoader>> _delayedDestroyedLoaders;
+	std::unique_ptr<Media::Audio::Instance> _audio;
+	QImage _logo;
+	QImage _logoNoMargin;
 
 };
