@@ -24,7 +24,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "platform/linux/linux_libnotify.h"
 #include "platform/linux/linux_libs.h"
 #include "lang.h"
-#include "core/task_queue.h"
+#include "base/task_queue.h"
 
 namespace Platform {
 namespace Notifications {
