@@ -25,7 +25,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "platform/win/windows_event_filter.h"
 #include "platform/win/windows_dlls.h"
 #include "mainwindow.h"
-#include "core/task_queue.h"
+#include "base/task_queue.h"
 
 #include <Shobjidl.h>
 #include <shellapi.h>
