@@ -23,7 +23,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "styles/style_profile.h"
 #include "styles/style_boxes.h"
 #include "ui/widgets/buttons.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "boxes/report_box.h"
 #include "mainwidget.h"
 #include "observer_peer.h"
