@@ -23,7 +23,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "styles/style_profile.h"
 #include "ui/widgets/labels.h"
 #include "ui/toast/toast.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "observer_peer.h"
 #include "mainwindow.h"
 #include "lang.h"
