@@ -28,9 +28,9 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "core/file_utilities.h"
 #include "ui/widgets/labels.h"
 #include "observer_peer.h"
-#include "boxes/confirmbox.h"
-#include "boxes/contactsbox.h"
-#include "boxes/photocropbox.h"
+#include "boxes/confirm_box.h"
+#include "boxes/contacts_box.h"
+#include "boxes/photo_crop_box.h"
 #include "lang.h"
 #include "apiwrap.h"
 #include "mainwidget.h"
