@@ -22,10 +22,10 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 
 #include "styles/style_settings.h"
 #include "lang.h"
-#include "boxes/connectionbox.h"
-#include "boxes/confirmbox.h"
-#include "boxes/aboutbox.h"
-#include "boxes/localstoragebox.h"
+#include "boxes/connection_box.h"
+#include "boxes/confirm_box.h"
+#include "boxes/about_box.h"
+#include "boxes/local_storage_box.h"
 #include "mainwindow.h"
 #include "ui/widgets/buttons.h"
 #include "ui/effects/widget_slide_wrap.h"
