@@ -20,7 +20,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
 #pragma once
 
-#include "core/observer.h"
+#include "base/observer.h"
 #include "settings/settings_block_widget.h"
 
 namespace Ui {
