@@ -29,9 +29,9 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "platform/platform_specific.h"
 #include "mainwindow.h"
 #include "application.h"
-#include "boxes/languagebox.h"
-#include "boxes/confirmbox.h"
-#include "boxes/aboutbox.h"
+#include "boxes/language_box.h"
+#include "boxes/confirm_box.h"
+#include "boxes/about_box.h"
 #include "core/file_utilities.h"
 #include "langloaderplain.h"
 #include "autoupdater.h"
