@@ -24,7 +24,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "lang.h"
 #include "ui/widgets/labels.h"
 #include "ui/effects/widget_slide_wrap.h"
-#include "boxes/usernamebox.h"
+#include "boxes/username_box.h"
 #include "boxes/change_phone_box.h"
 #include "observer_peer.h"
 #include "messenger.h"
