@@ -87,6 +87,7 @@ public:
 private slots:
 	void onBlockedUsers();
 	void onLastSeenPrivacy();
+	void onCallsPrivacy();
 	void onGroupsInvitePrivacy();
 	void onAutoLock();
 	void onShowSessions();
@@ -98,6 +99,7 @@ private:
 
 	object_ptr<Ui::LinkButton> _blockedUsers = { nullptr };
 	object_ptr<Ui::LinkButton> _lastSeenPrivacy = { nullptr };
+	object_ptr<Ui::LinkButton> _callsPrivacy = { nullptr };
 	object_ptr<Ui::LinkButton> _groupsInvitePrivacy = { nullptr };
 	object_ptr<LocalPasscodeState> _localPasscodeState = { nullptr };
 	object_ptr<Ui::WidgetSlideWrap<LabeledLink>> _autoLock = { nullptr };
