@@ -26,7 +26,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "lang.h"
 #include "storage/localstorage.h"
 #include "mainwindow.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "application.h"
 #include "ui/widgets/discrete_sliders.h"
 
