@@ -22,6 +22,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 
 #include "core/basic_types.h"
 #include "storage/file_download.h"
+#include "auth_session.h"
 
 namespace Window {
 namespace Theme {
@@ -31,6 +32,11 @@ struct Cached;
 
 namespace Local {
 
+struct StoredAuthSession {
+	AuthSessionData data;
+	float64 dialogsWidthRatio;
+};
+
 void start();
 void finish();
 
