@@ -21,8 +21,11 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #pragma once
 
 namespace Fonts {
-	void start();
-}
+
+void Start();
+QString GetOverride(const QString &familyName);
+
+} // namespace
 
 class Painter : public QPainter {
 public:
@@ -328,28 +331,23 @@ void myEnsureResized(QWidget *target);
 QPixmap myGrab(TWidget *target, QRect rect = QRect(), QColor bg = QColor(255, 255, 255, 0));
 QImage myGrabImage(TWidget *target, QRect rect = QRect(), QColor bg = QColor(255, 255, 255, 0));
 
-class SingleDelayedCall : public QObject {
-	Q_OBJECT
-
+class SingleQueuedInvokation : public QObject {
 public:
-	SingleDelayedCall(QObject *parent, const char *member) : QObject(parent), _member(member) {
+	SingleQueuedInvokation(base::lambda<void()> callback) : _callback(callback) {
 	}
 	void call() {
-		if (_pending.testAndSetOrdered(0, 1)) {
-			QMetaObject::invokeMethod(this, "makeDelayedCall", Qt::QueuedConnection);
-		}
-	}
-
-private slots:
-	void makeDelayedCall() {
-		if (_pending.testAndSetOrdered(1, 0)) {
-			QMetaObject::invokeMethod(parent(), _member);
+		if (_pending.testAndSetAcquire(0, 1)) {
+			InvokeQueued(this, [this] {
+				if (_pending.testAndSetRelease(1, 0)) {
+					_callback();
+				}
+			});
 		}
 	}
 
 private:
+	base::lambda<void()> _callback;
 	QAtomicInt _pending = { 0 };
-	const char *_member;
 
 };
 
