@@ -26,18 +26,12 @@ class MediaView;
 
 namespace Window {
 
-enum class GifPauseReason {
-	Any = 0,
-	InlineResults = (1 << 0),
-	SavedGifs = (1 << 1),
-	Layer = (1 << 2),
-	MediaPreview = (1 << 3),
-};
-Q_DECLARE_FLAGS(GifPauseReasons, GifPauseReason);
-Q_DECLARE_OPERATORS_FOR_FLAGS(GifPauseReasons);
-
 class TitleWidget;
 
+QImage LoadLogo();
+QImage LoadLogoNoMargin();
+QIcon CreateIcon();
+
 class MainWindow : public QWidget, protected base::Subscriber {
 	Q_OBJECT
 
@@ -79,6 +73,8 @@ public:
 	QWidget *filedialogParent();
 
 	void showRightColumn(object_ptr<TWidget> widget);
+	bool canExtendWidthBy(int addToWidth);
+	void tryToExtendWidthBy(int addToWidth);
 
 	virtual void updateTrayMenu(bool force = false) {
 	}
@@ -94,13 +90,6 @@ public:
 	}
 	virtual PeerData *ui_getPeerForMouseAction();
 
-	void enableGifPauseReason(GifPauseReason reason);
-	void disableGifPauseReason(GifPauseReason reason);
-	base::Observable<void> &gifPauseLevelChanged() {
-		return _gifPauseLevelChanged;
-	}
-	bool isGifPausedAtLeastFor(GifPauseReason reason) const;
-
 public slots:
 	bool minimizeToTray();
 	void updateGlobalMenu() {
@@ -122,6 +111,8 @@ protected:
 	virtual void clearWidgetsHook() {
 	}
 
+	virtual void updateWindowIcon();
+
 	virtual void stateChangedHook(Qt::WindowState state) {
 	}
 
@@ -179,6 +170,7 @@ private:
 	object_ptr<TWidget> _body;
 	object_ptr<TWidget> _rightColumn = { nullptr };
 
+	QIcon _icon;
 	QString _titleText;
 
 	object_ptr<QTimer> _isActiveTimer;
@@ -186,9 +178,6 @@ private:
 
 	object_ptr<MediaView> _mediaView = { nullptr };
 
-	GifPauseReasons _gifPauseReasons = { 0 };
-	base::Observable<void> _gifPauseLevelChanged;
-
 };
 
 } // namespace Window
