@@ -61,6 +61,8 @@ private:
 	using Notification = internal::Notification;
 	using HideAllButton = internal::HideAllButton;
 
+	QPixmap hiddenUserpicPlaceholder() const;
+
 	void doUpdateAll() override;
 	void doShowNotification(HistoryItem *item, int forwardedCount) override;
 	void doClearAll() override;
@@ -111,6 +113,8 @@ private:
 
 	Animation _demoMasterOpacity;
 
+	mutable QPixmap _hiddenUserpicPlaceholder;
+
 };
 
 namespace internal {
