@@ -24,7 +24,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "window/themes/window_theme_editor_block.h"
 #include "mainwindow.h"
 #include "storage/localstorage.h"
-#include "boxes/confirmbox.h"
+#include "boxes/confirm_box.h"
 #include "styles/style_window.h"
 #include "styles/style_settings.h"
 #include "styles/style_dialogs.h"
@@ -34,12 +34,12 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/checkbox.h"
 #include "ui/widgets/multi_select.h"
-#include "core/parse_helper.h"
-#include "core/task_queue.h"
-#include "core/zlib_help.h"
+#include "base/parse_helper.h"
+#include "base/task_queue.h"
+#include "base/zlib_help.h"
 #include "ui/toast/toast.h"
 #include "core/file_utilities.h"
-#include "boxes/editcolorbox.h"
+#include "boxes/edit_color_box.h"
 #include "lang.h"
 
 namespace Window {
