@@ -22,7 +22,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 
 #include "styles/style_window.h"
 #include "ui/effects/ripple_animation.h"
-#include "boxes/editcolorbox.h"
+#include "boxes/edit_color_box.h"
 #include "lang.h"
 
 namespace Window {
